"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.appendDotCjs = exports.cloneProgram = void 0;
const path_1 = require("path");
const typescript_1 = require("typescript");
const preventLoop_cjs_1 = require("./preventLoop.cjs");
const shouldMutateModuleSpecifier_cjs_1 = require("./shouldMutateModuleSpecifier.cjs");
function cloneProgram(mainProgram, overrideOptions, { debug }) {
    debug(' + before createProgram');
    let options = {
        ...mainProgram.getCompilerOptions(),
    };
    delete options.project;
    delete options.tsBuildInfoFile;
    delete options.plugins;
    options = {
        ...options,
        target: typescript_1.ScriptTarget.ES2018,
        ...overrideOptions,
        noEmitOnError: false,
        incremental: false,
        declarationMap: false,
        declaration: false,
        composite: false,
        module: typescript_1.ModuleKind.CommonJS,
    };
    const knownSources = [];
    for (const file of mainProgram.getSourceFiles()) {
        if (file.isDeclarationFile)
            continue;
        knownSources.push(file.fileName /*.replace(/\.tsx?/i, (m0: string) => `.cjs${m0}`)*/);
    }
    // debug('knownSources=%j', knownSources);
    const program = (0, typescript_1.createProgram)({
        rootNames: knownSources,
        options,
    });
    Object.defineProperty(program, preventLoop_cjs_1.selfCreatedProgram, {
        value: true,
        configurable: false,
        writable: false,
        enumerable: false,
    });
    debug(' + after createProgram');
    return program;
}
exports.cloneProgram = cloneProgram;
function appendDotCjs(program, console) {
    function writeFile(fileName, data, ...args) {
        const dir = (0, path_1.dirname)(fileName);
        const base = (0, path_1.basename)(fileName);
        const newBase = base.replace(/(?:\.jsx?)(\.map$|$)/i, '.cjs$1');
        console.debug('   * cjs write: %s/{%s => %s}', dir, base, newBase);
        if (base.endsWith('.map')) {
            const mapData = JSON.parse(data);
            if (mapData.file) {
                mapData.file = mapData.file.replace(/\.jsx?$/i, '.cjs');
            }
            else {
                console.error(`[dual-package] warning: sourcemap file ${dir}/${newBase} did not contains file.`);
            }
            data = JSON.stringify(mapData, null, 4);
        }
        return typescript_1.sys.writeFile(`${dir}/${newBase}`, data, ...args);
    }
    function addCjsExtension(transformationContext) {
        function visitNode(node) {
            if ((0, shouldMutateModuleSpecifier_cjs_1.shouldMutateModuleSpecifier)(node.getSourceFile().fileName, node, console, program)) {
                const moduleSpecifier = transformationContext.factory.createStringLiteral(`${node.moduleSpecifier.text}.cjs`);
                Object.assign(moduleSpecifier, { parent: node });
                Object.assign(node, { moduleSpecifier });
            }
            return node;
        }
        return (sourceFile) => {
            return (0, typescript_1.visitEachChild)(sourceFile, visitNode, transformationContext);
        };
    }
    // https://github.com/Jack-Works/commonjs-import.meta/blob/master/index.ts
    function updateMetaRequest(transformationContext) {
        // @ts-ignore
        const { factory: ts } = transformationContext;
        function visitNode(node) {
            if ((0, typescript_1.isPropertyAccessExpression)(node) &&
                (0, typescript_1.isMetaProperty)(node.expression) &&
                (0, typescript_1.idText)(node.expression.name) === 'meta' &&
                (0, typescript_1.idText)(node.name) === 'url') {
                return ts.createParenthesizedExpression(ts.createBinaryExpression(ts.createStringLiteral('file://'), typescript_1.SyntaxKind.PlusToken, ts.createIdentifier('__filename')));
            }
            else {
                return (0, typescript_1.visitEachChild)(node, visitNode, transformationContext);
            }
        }
        return (sourceFile) => {
            return (0, typescript_1.visitEachChild)(sourceFile, visitNode, transformationContext);
        };
    }
    return (sourceFile) => {
        program.emit(program.getSourceFile(sourceFile.fileName), writeFile, undefined, false, {
            before: [addCjsExtension, updateMetaRequest],
        });
    };
}
exports.appendDotCjs = appendDotCjs;
//# sourceMappingURL=append.cjs.js.map