import { idText, isMetaProperty, isPropertyAccessExpression, SyntaxKind, visitEachChild, } from 'typescript';
import { shouldMutateModuleSpecifier } from "./shouldMutateModuleSpecifier.js";
export function appendDotCjsDirect(program, transformationContext, debug) {
    function addCjsExtension(transformationContext) {
        function visitNode(node) {
            if (shouldMutateModuleSpecifier(node.getSourceFile().fileName, node, debug, program)) {
                const moduleSpecifier = transformationContext.factory.createStringLiteral(`${node.moduleSpecifier.text}.cjs`);
                Object.assign(moduleSpecifier, { parent: node });
                Object.assign(node, { moduleSpecifier });
            }
            return node;
        }
        return (sourceFile) => {
            return visitEachChild(sourceFile, visitNode, transformationContext);
        };
    }
    // https://github.com/Jack-Works/commonjs-import.meta/blob/master/index.ts
    function updateMetaRequest(transformationContext) {
        // @ts-ignore
        const { factory: ts } = transformationContext;
        function visitNode(node) {
            if (isPropertyAccessExpression(node) &&
                isMetaProperty(node.expression) &&
                idText(node.expression.name) === 'meta' &&
                idText(node.name) === 'url') {
                return ts.createParenthesizedExpression(ts.createBinaryExpression(ts.createStringLiteral('file://'), SyntaxKind.PlusToken, ts.createIdentifier('__filename')));
            }
            else {
                return visitEachChild(node, visitNode, transformationContext);
            }
        }
        return (sourceFile) => {
            return visitEachChild(sourceFile, visitNode, transformationContext);
        };
    }
    const a = addCjsExtension(transformationContext);
    const b = updateMetaRequest(transformationContext);
    return (sourceFile) => {
        return b(a(sourceFile));
    };
}
//# sourceMappingURL=append.cjs.heft.js.map