import { basename, dirname } from 'path';
import { createProgram, idText, isMetaProperty, isPropertyAccessExpression, ModuleKind, ScriptTarget, SyntaxKind, sys, visitEachChild, } from 'typescript';
import { selfCreatedProgram } from "./preventLoop.js";
import { shouldMutateModuleSpecifier } from "./shouldMutateModuleSpecifier.js";
export function cloneProgram(mainProgram, overrideOptions, { debug }) {
    debug(' + before createProgram');
    let options = {
        ...mainProgram.getCompilerOptions(),
    };
    delete options.project;
    delete options.tsBuildInfoFile;
    delete options.plugins;
    options = {
        ...options,
        target: ScriptTarget.ES2018,
        ...overrideOptions,
        noEmitOnError: false,
        incremental: false,
        declarationMap: false,
        declaration: false,
        composite: false,
        module: ModuleKind.CommonJS,
    };
    const knownSources = [];
    for (const file of mainProgram.getSourceFiles()) {
        if (file.isDeclarationFile)
            continue;
        knownSources.push(file.fileName /*.replace(/\.tsx?/i, (m0: string) => `.cjs${m0}`)*/);
    }
    // debug('knownSources=%j', knownSources);
    const program = createProgram({
        rootNames: knownSources,
        options,
    });
    Object.defineProperty(program, selfCreatedProgram, {
        value: true,
        configurable: false,
        writable: false,
        enumerable: false,
    });
    debug(' + after createProgram');
    return program;
}
export function appendDotCjs(program, console) {
    function writeFile(fileName, data, ...args) {
        const dir = dirname(fileName);
        const base = basename(fileName);
        const newBase = base.replace(/(?:\.jsx?)(\.map$|$)/i, '.cjs$1');
        console.debug('   * cjs write: %s/{%s => %s}', dir, base, newBase);
        if (base.endsWith('.map')) {
            const mapData = JSON.parse(data);
            if (mapData.file) {
                mapData.file = mapData.file.replace(/\.jsx?$/i, '.cjs');
            }
            else {
                console.error(`[dual-package] warning: sourcemap file ${dir}/${newBase} did not contains file.`);
            }
            data = JSON.stringify(mapData, null, 4);
        }
        return sys.writeFile(`${dir}/${newBase}`, data, ...args);
    }
    function addCjsExtension(transformationContext) {
        function visitNode(node) {
            if (shouldMutateModuleSpecifier(node.getSourceFile().fileName, node, console, program)) {
                const moduleSpecifier = transformationContext.factory.createStringLiteral(`${node.moduleSpecifier.text}.cjs`);
                Object.assign(moduleSpecifier, { parent: node });
                Object.assign(node, { moduleSpecifier });
            }
            return node;
        }
        return (sourceFile) => {
            return visitEachChild(sourceFile, visitNode, transformationContext);
        };
    }
    // https://github.com/Jack-Works/commonjs-import.meta/blob/master/index.ts
    function updateMetaRequest(transformationContext) {
        // @ts-ignore
        const { factory: ts } = transformationContext;
        function visitNode(node) {
            if (isPropertyAccessExpression(node) &&
                isMetaProperty(node.expression) &&
                idText(node.expression.name) === 'meta' &&
                idText(node.name) === 'url') {
                return ts.createParenthesizedExpression(ts.createBinaryExpression(ts.createStringLiteral('file://'), SyntaxKind.PlusToken, ts.createIdentifier('__filename')));
            }
            else {
                return visitEachChild(node, visitNode, transformationContext);
            }
        }
        return (sourceFile) => {
            return visitEachChild(sourceFile, visitNode, transformationContext);
        };
    }
    return (sourceFile) => {
        program.emit(program.getSourceFile(sourceFile.fileName), writeFile, undefined, false, {
            before: [addCjsExtension, updateMetaRequest],
        });
    };
}
//# sourceMappingURL=append.cjs.js.map