"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const util_1 = require("util");
const typescript_1 = tslib_1.__importDefault(require("typescript"));
const append_cjs_cjs_1 = require("./append.cjs.cjs");
const append_cjs_heft_cjs_1 = require("./append.cjs.heft.cjs");
const append_js_cjs_1 = require("./append.js.cjs");
const debug_cjs_1 = require("./debug.cjs");
const preventLoop_cjs_1 = require("./preventLoop.cjs");
let lastProgram;
let cachedTransformer;
function makeLogger(logger, verbose) {
    return {
        debug(msg, ...args) {
            if (verbose) {
                logger.terminal.writeLine((0, util_1.format)(msg, ...args));
            }
            else {
                logger.terminal.writeVerboseLine((0, util_1.format)(msg, ...args));
            }
        },
        error(msg) {
            logger.terminal.writeWarningLine(msg);
        },
    };
}
function typescriptTransformerDualPackage(program, pluginOptions) {
    var _a;
    const verbose = pluginOptions.verbose || ((_a = process.env.NODE_DEBUG) === null || _a === void 0 ? void 0 : _a.includes('tramsform')) || false;
    const logger = pluginOptions.logger ? makeLogger(pluginOptions.logger, verbose) : (0, debug_cjs_1.getDebug)(verbose);
    if (preventLoop_cjs_1.selfCreatedProgram in program) {
        return () => {
            (node) => node;
        };
    }
    if (program === lastProgram) {
        return cachedTransformer;
    }
    lastProgram = program;
    if (pluginOptions.logger) {
        let transformer;
        const mdl = program.getCompilerOptions().module;
        logger.debug('emit program type ' + typescript_1.default.ModuleKind[mdl]);
        if (mdl === typescript_1.default.ModuleKind.CommonJS) {
            if (pluginOptions.cjs === false) {
                logger.debug('config cjs is false');
                transformer = noop;
            }
            else {
                const ext = pluginOptions.cjs || '.cjs';
                logger.debug('creating program for %s', ext);
                transformer = function transformer(transformationContext) {
                    logger.debug('[trans] NewContext');
                    return (0, append_cjs_heft_cjs_1.appendDotCjsDirect)(program, transformationContext, logger);
                };
            }
        }
        else if (mdl >= typescript_1.default.ModuleKind.ES2015 && mdl <= typescript_1.default.ModuleKind.ESNext) {
            if (pluginOptions.mjs === false) {
                logger.debug('config mjs is false');
                transformer = noop;
            }
            else {
                const ext = pluginOptions.mjs || '.js';
                logger.debug('creating program for %s', ext);
                transformer = function transformer(transformationContext) {
                    logger.debug('[trans] NewContext');
                    return (0, append_js_cjs_1.appendDotJs)(program, transformationContext, logger);
                };
            }
        }
        else {
            logger.debug('creating noop program');
            transformer = noop;
        }
        return transformer;
    }
    else {
        const shadowCompilerOptions = pluginOptions.compilerOptions || {};
        shadowCompilerOptions.declaration = false;
        shadowCompilerOptions.declarationMap = false;
        // console.error('[dual-package]: ttsc: creating shadow program!');
        const shadowProgram = (0, append_cjs_cjs_1.cloneProgram)(program, shadowCompilerOptions, logger);
        const appendDotCjsCall = (0, append_cjs_cjs_1.appendDotCjs)(shadowProgram, logger);
        return (cachedTransformer = function transformer(transformationContext) {
            logger.debug('[trans] NewContext');
            const appendDotJsCall = (0, append_js_cjs_1.appendDotJs)(program, transformationContext, logger);
            return (sourceFile) => {
                appendDotCjsCall(sourceFile);
                return appendDotJsCall(sourceFile);
            };
        });
    }
}
exports.default = typescriptTransformerDualPackage;
function noop() {
    return (s) => s;
}
//# sourceMappingURL=register.js.map